@@ -210,53 +210,201 @@ class CSP(components.ProcessingMethod):
 
     Attributes
     ----------
-    w: np.array
-        Mixing matrix (spatial filters are stored in columns).
-    eigenvalues:  np.array
+    filters : {(…, M, M) numpy.ndarray, (…, M, M) matrix}
+            Mixing matrix (spatial filters are stored in columns).
+    eigenvalues : (…, M) numpy.ndarray
         Eigenvalues of w.
-    a: np.array
-    De-mixing matrix (activation patterns are stored in columns).
+    patterns : numpy.ndarray
+        De-mixing matrix (activation patterns are stored in columns).
     """
 
-    def __init__(self):
-        self.w = None  # Mixing matrix (spatial filters)
-        self.eigenvalues = None  # Eigenvalues
-        self.a = None  # De-mixing matrix
+    def __init__(self, n_filters=4):
+        """
+        n_filter : int or None
+            Number of most discriminant CSP filters to decompose the signal
+            into (must be less or equal to the number of channels in your
+            signal).
+            If int it will return that number of filters.
+            If None it will return all calculated filters."""
 
-    def fit(self, x1, x2):
-        """Common Spatial Patterns (CSP) in its most basic form.
+        self.filters = None  # Mixing matrix (spatial filters)
+        self.eigenvalues = None  # Eigenvalues
+        self.patterns = None  # De-mixing matrix
+        self.n_filters = n_filters
+
+    def fit(self, X, y):
+        """Train Common Spatial Patterns (CSP) filters
+
+            Train Common Spatial Patterns (CSP) filters with support to >2
+            classes based on support multiclass CSP by means of approximate
+            joint diagonalization. In this case, the spatial filter selection
+            is achieved according to [1].
+
+            Parameters
+            ----------
+            X : numpy.ndarray, [n_trials, samples, channels]
+                Epoched data of shape (n_trials, samples, channels)
+
+            y : numpy.ndarray, [n_trials,]
+                Labels for epoched data of shape (n_trials,)
+
+            References
+            ----------
+            [1] Grosse-Wentrup, Moritz, and Martin Buss. "Multiclass common
+            spatial patterns and information theoretic feature extraction."
+            Biomedical Engineering, IEEE Transactions on 55, no. 8 (2008):
+            1991-2000.
+        """
+        n_classes = np.unique(y)
+        # Covariance matrices
+        cov = []
+        for c in n_classes:
+            cov.append(np.cov(X[y == c].reshape(-1, X.shape[-1]).T))
+        cov = np.array(cov)
+        # Classic implementation for 2-class
+        if len(n_classes) == 2:
+            # Solve the eigenvalue problem
+            eigenvalues, filters = slinalg.eigh(cov[0], cov[0] + cov[1])
+
+            # Indexes for sorting eigenvectors (w)
+            ix_sorted = np.argsort(np.abs(eigenvalues - 0.5))[::-1]
+        # Implementation for >2-classes
+        elif len(n_classes) > 2:
+            # Approximate joint diagonalization based on jacobi angle
+            filters, d = self.adj_pham(x=cov)
+            filters = filters.T
+            # Normalization
+            # Mean covariance
+            cmean = np.average(cov, axis=0)
+            for i in range(filters.shape[1]):
+                temp = np.dot(np.dot(filters[:, i].T, cmean), filters[:, i])
+                filters[:, i] /= np.sqrt(temp)
+            # We calculate the probability of each class
+            info = []
+            prob_class = [np.mean(y == c) for c in n_classes]
+            for j in range(filters.shape[1]):
+                patterns, b = 0, 0
+                for i, c in enumerate(n_classes):
+                    temp = np.dot(np.dot(filters[:, j].T, cov[i]), filters[:, j])
+                    patterns += prob_class[i] * np.log(np.sqrt(temp))
+                    b += prob_class[i] * (temp ** 2 - 1)
+                mutual_info = - (patterns + (3.0 / 16) * (b ** 2))
+                info.append(mutual_info)
+            eigenvalues = info
+            # Indexes for sorting eigenvectors (w)
+            ix_sorted = np.argsort(info)[::-1]
+        else:
+            raise ValueError("Number of classes must be  >= 2")
+        # Sort eigenvectors (w)
+        filters = filters[:, ix_sorted]
+        eigenvalues = [eigenvalues[i] for i in ix_sorted]
+        eigenvalues = np.diag(eigenvalues)
+        # Activation spatial patterns
+        patterns = slinalg.pinv2(filters).T
+
+        # Attribute storing of number of filters
+        self.filters = filters[:, :self.n_filters].T
+        self.eigenvalues = eigenvalues
+        self.patterns = patterns[:, :self.n_filters].T
+
+    def project(self, X):
+        """ This method projects the input data X with the spatial filters W
 
         Parameters
         ----------
-        x1: np.array
-            Positive class (dimensions [channels x samples])
-        x2: np.array
-            Negative class (dimensions [channels x samples])
+        X : numpy.ndarray [n_trials, samples, channels]
+            Input data (dimensions [n_trials, samples, channels])
+
+        Returns
+        -------
+        numpy.ndarray [n_trials, n_filters, samples]
+            Array with the epochs of signal projected in the CSP space
         """
-
-        # Covariance matrices
-        s1 = np.cov(x1)
-        s2 = np.cov(x2)
-
-        # Solve the eigenvalue problem
-        l, w = slinalg.eig(s1, s1 + s2)
-        eigenvalues = np.diag(l)
-        a = slinalg.pinv2(w).T
-
-        # Attribute storing
-        self.w = w
-        self.eigenvalues = eigenvalues
-        self.a = a
-
-    def apply(self, x):
-        """ This method projects the input data X with the spatial filters W.
-            :param x: Input data (dimensions [channels x samples]
-
-            :return Filtered signal.
-        """
-        if self.w is None:
+        if self.filters is None:
             raise Exception("CSP must be fitted first")
-        return np.matmul(self.w.T, x)
+        return np.matmul(self.filters, X.transpose((0, 2, 1)))
+
+    def adj_pham(self, x, eps=1e-6, n_iter_max=15):
+        """Approximate joint diagonalization based on pham's algorithm.
+            This is a direct implementation of the PHAM's AJD algorithm [1].
+            Extracted from pyriemann module:
+            http://github.com/alexandrebarachant/pyRiemann
+
+            Parameters
+            ----------
+            x : ndarray, shape (n_trials, n_channels, n_channels)
+                A set of covariance matrices to diagonalize
+            eps : float (default 1e-6)
+                tolerance for stopping criterion.
+            n_iter_max : int (default 15)
+                The maximum number of iteration to reach convergence.
+
+            Returns
+            -------
+            v : numpy.ndarray, [n_channels, n_channels]
+                Diagonalizer
+            d : numpy.ndarray, [n_trials, n_channels, n_channels]
+                Set of quasi diagonal matrices
+
+            References
+            ----------
+            [1] Pham, Dinh Tuan. "Joint approximate diagonalization of positive
+            definite Hermitian matrices." SIAM Journal on Matrix Analysis and
+            Applications 22, no. 4 (2001): 1136-1152.
+            """
+        n_epochs = x.shape[0]
+
+        # Reshape input matrix
+        a = np.concatenate(x, axis=0).T
+
+        # Init variables
+        n_times, n_m = a.shape
+        v = np.eye(n_times)
+        epsilon = n_times * (n_times - 1) * eps
+
+        for it in range(n_iter_max):
+            decr = 0
+            for ii in range(1, n_times):
+                for jj in range(ii):
+                    Ii = np.arange(ii, n_m, n_times)
+                    Ij = np.arange(jj, n_m, n_times)
+
+                    c1 = a[ii, Ii]
+                    c2 = a[jj, Ij]
+
+                    g12 = np.mean(a[ii, Ij] / c1)
+                    g21 = np.mean(a[ii, Ij] / c2)
+
+                    omega21 = np.mean(c1 / c2)
+                    omega12 = np.mean(c2 / c1)
+                    omega = np.sqrt(omega12 * omega21)
+
+                    tmp = np.sqrt(omega21 / omega12)
+                    tmp1 = (tmp * g12 + g21) / (omega + 1)
+                    tmp2 = (tmp * g12 - g21) / max(omega - 1, 1e-9)
+
+                    h12 = tmp1 + tmp2
+                    h21 = np.conj((tmp1 - tmp2) / tmp)
+
+                    decr += n_epochs * (g12 * np.conj(h12) + g21 * h21) / 2.0
+
+                    tmp = 1 + 1.j * 0.5 * np.imag(h12 * h21)
+                    tmp = np.real(tmp + np.sqrt(tmp ** 2 - h12 * h21))
+                    tau = np.array([[1, -h12 / tmp], [-h21 / tmp, 1]])
+
+                    a[[ii, jj], :] = np.dot(tau, a[[ii, jj], :])
+                    tmp = np.c_[a[:, Ii], a[:, Ij]]
+                    tmp = np.reshape(tmp, (n_times * n_epochs, 2), order='F')
+                    tmp = np.dot(tmp, tau.T)
+
+                    tmp = np.reshape(tmp, (n_times, n_epochs * 2), order='F')
+                    a[:, Ii] = tmp[:, :n_epochs]
+                    a[:, Ij] = tmp[:, n_epochs:]
+                    v[[ii, jj], :] = np.dot(tau, v[[ii, jj], :])
+            if decr < epsilon:
+                break
+        d = np.reshape(a, (n_times, -1, n_times)).transpose(1, 0, 2)
+        return v, d
 
     def to_dict(self):
         return self.__dict__
@@ -264,9 +412,10 @@ class CSP(components.ProcessingMethod):
     @staticmethod
     def from_dict(dict_data):
         csp = CSP()
-        csp.w = dict_data['w']
+        csp.filters = dict_data['filters']
         csp.eigenvalues = dict_data['eigenvalues']
-        csp.a = dict_data['a']
+        csp.patterns = dict_data['patterns']
+        csp.n_filters = dict_data['n_filters']
         return csp
 
 
